/***************************************************************************************************
*          C# sample for the usage of class TopoART_R (approximation of a sine function)           *
****************************************************************************************************
*                           Created by Marko Tscherepanow, 11 February 2012                        *
***************************************************************************************************/

// Compile and run from the console: dotnet run -p TopoART-R_sample1.csproj

using System;
using System.IO;
using System.Reflection;
using LibTopoART;

namespace LibTopoART_samples
{
	/// <summary>
	/// Regression sample using TopoART-R. (simplified version) [C#]
	/// <para>
	/// This sample trains a TopoART-R network with 100 points sampled from a sine function.
	/// Then, sine values are predicted for 25 random values.
	/// </para>
	/// <para>The predicted results can be visualised using the R script <c>ShowTopoARTRResults.R</c> provided 
	/// in the subfolder <c>R</c>.
	/// </para>
	/// </summary>
	class TopoART_R_sample1
	{
		static void Main() {
			long i;

			// Destination directory for the regression results
			const string ResultPath = "../../../../../results/regression/";

			// Input and output dimensions
			const long i_len = 1;	// number of independent variables (input dimensions)
			const long d_len = 1;	// number of dependent variables (output dimensions)

			// Training data
			const long training_sample_num	=	100; 
			decimal[][] training_samples	=	new decimal[training_sample_num][];

			// Test data
			const long test_sample_num		=	25; 
					
			// Set working directory to assembly directory
            Directory.SetCurrentDirectory(Path.GetDirectoryName(new Uri(Assembly.GetEntryAssembly().CodeBase).LocalPath));

			// Create training samples (sine function)
			// In this sample, input vectors holding the independent and the dependent variable are applied.
			for(i = 0; i < training_sample_num; ++i) {
				training_samples[i]		=	new decimal[i_len + d_len];

				// Set the independent variable
				training_samples[i][0]	=	(decimal)i * (decimal)Math.PI * 2.0m / (decimal)(training_sample_num - 1);

				// Set the dependent variable
				training_samples[i][1]	=	((decimal)Math.Sin((double)training_samples[i][0]) + 1.0m) / 2.0m;

				// Normalisation of the independent variable
				training_samples[i][0]	/=	((decimal)Math.PI * 2.0m);													
			}
			
			// Create TopoART-R network using rho_a = 0.99 and default values for the remaining parameters
			TopoART_R tar = new TopoART_R(i_len, d_len, 2, 0.99m);

			// Create file for the results and write the training data
			using(TextWriter writer = new StreamWriter(File.Open(ResultPath + "TopoART-R_sine_regression_results.csv", 
				FileMode.Create)))
			{
				writer.WriteLine("   {0,6:D},   {1,6:D}", training_sample_num, test_sample_num);

				// Training with a common array for the independent variables and the dependent variables 
				Console.WriteLine("Train TopoART-R network with {0} samples", training_sample_num);
				for(i = 0; i < training_sample_num; ++i) {
					tar.Learn(training_samples[i]);

					// Write denormalised training data
					writer.WriteLine("   {0},   {1}", 
	                 	(training_samples[i][0] * (decimal)Math.PI * 2.0m).ToString(" 0.000;-0.000", System.Globalization.CultureInfo.InvariantCulture), 
	                 	((training_samples[i][1] - 0.5m) * 2.0m).ToString(" 0.000;-0.000", System.Globalization.CultureInfo.InvariantCulture));
				}

				// Prediction without using a mask vector (all independent variables are required to be present) 
				decimal[] prediction;
				decimal[] random_i_vec = new decimal[i_len];
				Random rnd = new Random();
				Console.WriteLine("Predict sine function for 25 random values");
				for(i = 0; i < test_sample_num; ++i) {
					random_i_vec[0] = (decimal)rnd.NextDouble();
					prediction		= tar.Predict(random_i_vec);
					writer.WriteLine("   {0},   {1}",
						(random_i_vec[0] * (decimal)Math.PI * 2.0m).ToString(" 0.000;-0.000", System.Globalization.CultureInfo.InvariantCulture), 
						((prediction[0] - 0.5m) * 2.0m).ToString(" 0.000;-0.000", System.Globalization.CultureInfo.InvariantCulture)); 
				}
			}
		}
	}
}